import QtQuick 2.12
//import QtQuick.Window 2.12
import MuseScore 3.0

MuseScore {
    menuPath: "Plugins.NoteNumber"
    description: "Note namber"
    version: "3.0"
    pluginType: "dialog"
    requiresScore: false
//Window {
    id: window
    width: 480
    height: 480
    visible: true
    property var keys: [0,1,0,1,0,0,1,0,1,0,1,0]
    property var noteName: ["F","C","G","D","A","E","B"]
    property var altSign: ["♭","♯"]
    Component.onCompleted: { start() }

    ListView {
        id: listView
        anchors.fill: parent
        anchors.rightMargin: 120
        anchors.bottomMargin: 0
        anchors.topMargin: 0

        headerPositioning: ListView.OverlayHeader
        header: Rectangle {
            id: headerItem
            width: listView.width
            height:30
            Row {
                spacing: 5
                padding: 5
                Repeater{
                    model: ["#", "name", "oktave", "pitch"]
                    Rectangle{
                        width: (listView.width - 25) / 4
                        height:30
                        color: "#e0e0e0"
                        Text {
                            anchors.fill: parent
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                            font.capitalization: Font.AllUppercase
                            text: modelData
                        }
                    }
                }
            }
        }
        delegate: Item {
            width: listView.width
            height: 30
            Rectangle{
                anchors.fill: parent
                color: "Ivory"
                Row {
                    id: row1
                    spacing: 5
                    padding: 5
                    Repeater{
                        model: listView.model[index]
                        Rectangle {
                            width: (listView.width - 25) / 4
                            height: 30
                            Text {
                                anchors.fill: parent
                                horizontalAlignment: Text.AlignHCenter
                                verticalAlignment: Text.AlignVCenter
                                text: modelData
                            }
                        }
                    }
                }
            }
        }
    }

    Column {
        id: column
        //width: 120
        spacing: 10
        padding: 10
        anchors.right: parent.right

        Rectangle {
            id: rectangle
            width: 100
            height: 40
            color: "#d9d9d9"

            MouseArea {
                id: mouseArea
                anchors.fill: parent

                onClicked: { 
                  curScore.startCmd();
                  setNoteText(); 
                  curScore.endCmd();
                }
            }

            Text {
                id: textStart
                text: qsTr("Start")
                anchors.fill: parent
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
            }
        }
    }

    function getTpcIndex(tpc) {
        var alteration = -2;
        var tpcIndex = tpc;
        while (tpcIndex > 5) {
            tpcIndex -= 7;
            alteration++;
        }
        tpcIndex++;
        return [tpcIndex, alteration];
    }

    function getNoteName(tpc) {
        var tpcIndex = getTpcIndex(tpc);
        var name = noteName[tpcIndex[0]];
        for (var alteration = tpcIndex[1]; alteration !== 0; alteration < 0 ? alteration++ : alteration--) {
            name += altSign[alteration < 0 ? 0 : 1];
        }
        return name;
    }

    function getNextItem (value, step, limit, size) {
        var result = value + step;
        if (result > limit) { result -= size; }
        if (result < (limit - size)) { result += size; }
        return result;
    }

    function getKeyIndex(pitch) {
        var keyIndex = pitch;
        while (keyIndex > 11) { keyIndex -= 12 }
        return keyIndex;
    }

    function getNumber(pitch) {
        var keyIndex = getKeyIndex(pitch);
        var alteration = 0;
        var countWhiteKeys = 0;
        for (var i = 0; i <= keyIndex; i++) {
            alteration = keys[i];
            if (alteration === 0) { countWhiteKeys++}
        }

        var result = getNextItem (countWhiteKeys, 0, 7, 7);
        if (alteration > 0) {
            countWhiteKeys++;
            var nextItem = getNextItem (countWhiteKeys, 1, 7, 7);
            result += altSign[1] + "/" + nextItem + altSign[0];
        }
        return "" + result;
    }

    function getOktave(pitch) {
        var result = 0;
        var i = -1;
        while(i < pitch) {
            i += 12;
            result++;
        }
        return result;
    }

    function start() {
    }

    function getNumberArr(pitch, tpc) {
        return [getNumber(pitch), getNoteName(tpc), getOktave(pitch), "" + pitch];
    }

    function getModelKeys() {
        var modelArr = [];
        for (var i = 0; i < 12; i++) {
            modelArr[i] = getLetter(i);
        }
        comboBoxKey.model = modelArr;
    }

    function getNoteText(notes, listViewModel) {
        var noteText = "";
        for (var i = 0; i < notes.length; i++) {
            var note = notes[i];
            if ( i > 0 )
                noteText  = "\n" + noteText;
            if (typeof note.pitch === "undefined")
                continue

            var itemModel = getNumberArr(note.pitch, note.tpc);
            listViewModel.push(itemModel);
            noteText  = itemModel[0]+ ":" + itemModel[1] + noteText;

        }

        var textHord = newElement(Element.STAFF_TEXT);
        textHord.text = noteText;
        return textHord;
    }


    function setNoteText() {
        var listViewModel = [];
        var noteText;
        var cursor = curScore.newCursor();
        for (var staff = 0; staff < curScore.nstaves; staff++) {
            for (var voice = 0; voice < 4; voice++) {
                cursor.rewind(0);
                cursor.voice = voice;
                cursor.staffIdx = staff;
                while (cursor.element) {
                    if (cursor.element.type === Element.CHORD) {
                        for (var i = 0; i < cursor.element.graceNotes.length; i++) {
                            noteText = getNoteText(cursor.element.graceNotes[i].notes, listViewModel)
                            cursor.add(noteText);
                        }
                        noteText = getNoteText(cursor.element.notes, listViewModel)
                        cursor.add(noteText);
                    }
                    cursor.next();
                }
            }
        }
        listView.model = listViewModel;
    }


}
